/*
 * at_commands.h
 *
 *  Created on: May 18, 2021
 *      Author: pratham_ovalekar
 */

#ifndef INC_AT_COMMANDS_H_
#define INC_AT_COMMANDS_H_

#include <stdbool.h>
#include "stm32l4r5xx_uart.h"


typedef enum {
    AT_MODE_UNKNOWN,
    AT_MODE_DATA,
    AT_MODE_AT,
    AT_MODE_EDM
} AT_mode_t;

#ifndef ucxhAT_INBUF_SIZE
  /** Default buffer size
   * @def ucxhAT_INBUF_SIZE
   */
  #define ucxhAT_INBUF_SIZE  2048
#endif


// Decimal equivalent of characters
#define CR	13		// Carriage return
#define NL	10		// NL Line feed, new Line


/** Line delimiter for AT-commands in AT mode */
#define ucxhAT_LINE_DELIMITER "\r\n"
/** Line delimiter for AT-commands in Extended Data Node (EDM) */
#define ucxhEDM_LINE_DELIMITER "\r"


//
#define EOL_REACHED			1
#define EOL_NOT_REACHED		0



# define AT_COMMAND			"AT\r\n"			// Attention AT

// AT Echo commands
# define AT_ECHO_OFF		"ATE0\r\n"
# define AT_ECHO_ON			"ATE1\r\n"

# define AT_MANU_IDENTI		"AT+CGMI\r\n"		// Manufacturer identification
# define AT_MODEL_IDENTI	"AT+CGMM\r\n"		// Model identification
# define AT_SOFT_VER_ID		"AT+CGMR\r\n"		// Software version identification
# define AT_SERIAL_NUM		"AT+CGSN\r\n"		// Serial number


# define AT_CMD_ENTER_DATA_MODE "ATO1\r\n"
# define AT_CMD_LEAVE_DATA_MODE "+++"

// Module start mode +UMSM



# define AT_DISCOV_Q			"AT+UBTDM?\r\n"
# define AT_DISCOV				"AT+UBTDM=3\r\n"

# define AT_CONECT_Q			"AT+UBTCM?\r\n"
# define AT_CONECT				"AT+UBTCM=2\r\n"	// 2 (factory default): GAP connectable mode


//  Pairing mode +UBTPM
# define AT_PAIRING_MODE_Q		"AT+UBTPM?\r\n"		// Reads the pairing mode.
# define AT_PAIRING_MODE_NO		"AT+UBTPM=1\r\n"	// GAP non-pairing mode
# define AT_PAIRING_MODE_YES	"AT+UBTPM=2\r\n"	// GAP pairing mode



// Security mode +UBTSM
# define AT_SECURITY_MODE_Q		"AT+UBTSM?\r\n"
# define AT_SECURITY_MODE_1		"AT+UBTSM=1\r\n"		// (factory default): Security Disabled.
# define AT_SECURITY_MODE_2		"AT+UBTSM=2\r\n"		// Security Enabled - Just Works


// Bluetooth low energy role
# define AT_BLE_ROLE_Q			"AT+UBTLE?\r\n"			// Disabled.
# define AT_BLE_ROLE_0			"AT+UBTLE=0\r\n"		// BLE Central
# define AT_BLE_ROLE_1			"AT+UBTLE=1\r\n"		// BLE Peripheral.
# define AT_BLE_ROLE_2			"AT+UBTLE=2\r\n"		// BLE Simultaneous Peripheral and Central.

#define AT_DTS_MODE_Q	"AT&S?\r\n"
#define AT_DTS_MODE_1	"AT&S1\r\n"
#define AT_DTS_MODE_2	"AT&S2\r\n"

void sendATCommand(  char *pTxBuffer );

void parseATCommandResponse( void );

/**
 * Returns the next line from the serial port, excluding any leading LF, without any trailing CR or CR LF
 * @returns NULL if the serial buffer is empty.
 * @param [out] atMaxBufferSize Returns the total number of bytes in the serial buffer. May vary
 * from invocation to invocation, depending on how many lines that have been read into the buffer.
 * \bug Will treat a data in the serial buffer as a complete line, even if it doesn't end in CR or CR LF
 * \protected
 */
uint8_t *atReadLine( size_t *atMaxBufferSize );

int readBluetoothQueue(uint8_t *outbuffer, uint8_t *eolR );

bool enterAtMode(void);

bool enterDataMode(void);

AT_mode_t getCurrentATMode(void);




#endif /* INC_AT_COMMANDS_H_ */
