

#ifndef __STM32L4R5xxEXTD_H
#define __STM32L4R5xxEXTD_H


#include "stm32l4r5xx.h"

/*
 * Clock Enable Macros for GPIOx peripherals
 * All the GPIO lies on the AHB2
 * SO enabling these: AHB2 peripheral clock enable register RCC_AHB2ENR
 */
#define GPIOA_PCLK_EN() (RCC->AHB2ENR |= ( 1 << 0 ))
#define GPIOB_PCLK_EN() (RCC->AHB2ENR |= ( 1 << 1 ))
#define GPIOC_PCLK_EN() (RCC->AHB2ENR |= ( 1 << 2 ))
#define GPIOD_PCLK_EN() (RCC->AHB2ENR |= ( 1 << 3 ))
#define GPIOE_PCLK_EN() (RCC->AHB2ENR |= ( 1 << 4 ))
#define GPIOF_PCLK_EN() (RCC->AHB2ENR |= ( 1 << 5 ))
#define GPIOG_PCLK_EN() (RCC->AHB2ENR |= ( 1 << 6 ))
#define GPIOH_PCLK_EN() (RCC->AHB2ENR |= ( 1 << 7 ))
#define GPIOI_PCLK_EN() (RCC->AHB2ENR |= ( 1 << 8 ))


/*
 * Clock Enable Macros for I2Cx peripherals
 * I2C lies on the APB1
 * Using these registers: APB1 peripheral clock enable register 1 (RCC_APB1ENR1) &
 * APB1 peripheral clock enable register 2 (RCC_APB1ENR2) to Enable the I2C clock
 */

#define I2C1_PCLK_EN() (RCC->APB1ENR1 |= ( 1 << 21 ))
#define I2C2_PCLK_EN() (RCC->APB1ENR1 |= ( 1 << 22 ))
#define I2C3_PCLK_EN() (RCC->APB1ENR1 |= ( 1 << 23 ))
#define I2C4_PCLK_EN() (RCC->APB1ENR2 |= ( 1 << 1  ))

/*
 * Clock Enable Macros for SPIx peripherals
 * SPI1 on APB2 || SPI2 & SPI3 on APB1
 * Using these registers: APB1 peripheral clock enable register 1 (RCC_APB1ENR1) &
 * APB2 peripheral clock enable register (RCC_APB2ENR) to Enable the SPI clock
 */

#define SPI1_PCLK_EN() (RCC->APB2ENR |= ( 1 << 12 ))
#define SPI2_PCLK_EN() (RCC->APB1ENR1 |= ( 1 << 14 ))
#define SPI3_PCLK_EN() (RCC->APB1ENR1 |= ( 1 << 15 ))

/*
 * Clock Enable Macros for UART peripherals
 * USART1 on APB2 || USART2, USART3, UART4 & UART5 on APB1
 * Using these registers: APB1 peripheral clock enable register 1 (RCC_APB1ENR1) &
 * APB2 peripheral clock enable register (RCC_APB2ENR) to Enable the SPI clock
 */

#define USART1_PCLK_EN() (RCC->APB2ENR  |= ( 1 << 14 ))
#define USART2_PCLK_EN() (RCC->APB1ENR1 |= ( 1 << 17 ))
#define USART3_PCLK_EN() (RCC->APB1ENR1 |= ( 1 << 18 ))
#define UART4_PCLK_EN()  (RCC->APB1ENR1 |= ( 1 << 19 ))
#define UART5_PCLK_EN()  (RCC->APB1ENR1 |= ( 1 << 20 ))

/*
 * Clock Enable Macros for SYSCONFG peripheral
 * Connected to APB2
 */
#define SYSCONFG_PCLK_EN() (RCC->APB2ENR  |= ( 1 << RCC_APB2RSTR_SYSCFGRST_Pos ))


/*-------------------------------------------------------------------------------------*/


/*
 * Clock disable Macros for GPIOx peripherals
 * All the GPIO lies on the AHB2
 * SO enabling these: AHB2 peripheral clock enable register RCC_AHB2ENR
 */
#define GPIOA_PCLK_DI() (RCC->AHB2ENR &= ~( 1 << 0 ))
#define GPIOB_PCLK_DI() (RCC->AHB2ENR &= ~( 1 << 1 ))
#define GPIOC_PCLK_DI() (RCC->AHB2ENR &= ~( 1 << 2 ))
#define GPIOD_PCLK_DI() (RCC->AHB2ENR &= ~( 1 << 3 ))
#define GPIOE_PCLK_DI() (RCC->AHB2ENR &= ~( 1 << 4 ))
#define GPIOF_PCLK_DI() (RCC->AHB2ENR &= ~( 1 << 5 ))
#define GPIOG_PCLK_DI() (RCC->AHB2ENR &= ~( 1 << 6 ))
#define GPIOH_PCLK_DI() (RCC->AHB2ENR &= ~( 1 << 7 ))
#define GPIOI_PCLK_DI() (RCC->AHB2ENR &= ~( 1 << 8 ))


/*
 * Clock Disable Macros for I2Cx peripherals
 * I2C lies on the APB1
 * Using these registers: APB1 peripheral clock enable register 1 (RCC_APB1ENR1) &
 * APB1 peripheral clock enable register 2 (RCC_APB1ENR2) to Enable the I2C clock
 */

#define I2C1_PCLK_DI() (RCC->APB1ENR1 &= ~( 1 << 21 ))
#define I2C2_PCLK_DI() (RCC->APB1ENR1 &= ~( 1 << 22 ))
#define I2C3_PCLK_DI() (RCC->APB1ENR1 &= ~( 1 << 23 ))
#define I2C4_PCLK_DI() (RCC->APB1ENR2 &= ~( 1 << 1  ))

/*
 * Clock Disable Macros for SPIx peripherals
 * SPI1 on APB2 || SPI2 & SPI3 on APB1
 * Using these registers: APB1 peripheral clock enable register 1 (RCC_APB1ENR1) &
 * APB2 peripheral clock enable register (RCC_APB2ENR) to Enable the SPI clock
 */

#define SPI1_PCLK_DI() (RCC->APB2ENR  &= ~( 1 << 12 ))
#define SPI2_PCLK_DI() (RCC->APB1ENR1 &= ~( 1 << 14 ))
#define SPI3_PCLK_DI() (RCC->APB1ENR1 &= ~( 1 << 15 ))

/*
 * Clock disable Macros for UART peripherals
 * USART1 on APB2 || USART2, USART3, UART4 & UART5 on APB1
 * Using these registers: APB1 peripheral clock enable register 1 (RCC_APB1ENR1) &
 * APB2 peripheral clock enable register (RCC_APB2ENR) to Enable the SPI clock
 */

#define USART1_PCLK_DI() (RCC->APB2ENR  &= ~( 1 << 14 ))
#define USART2_PCLK_DI() (RCC->APB1ENR1 &= ~( 1 << 17 ))
#define USART3_PCLK_DI() (RCC->APB1ENR1 &= ~( 1 << 18 ))
#define UART4_PCLK_DI()  (RCC->APB1ENR1 &= ~( 1 << 19 ))
#define UART5_PCLK_DI()  (RCC->APB1ENR1 &= ~( 1 << 20 ))

/*
 * Clock diable Macros for SYSCONFG peripheral
 * Connected to APB2
 */
#define SYSCONFG_PCLK_DI() (RCC->APB2ENR  &= ~( 1 << 0 ))


/*
 * Macros to reset GPIOx peripherals
 */

#define GPIOA_REG_RESET()	do{( RCC->AHB2RSTR |= ( 1 << 0 ) ); ( RCC->AHB2RSTR &= ~( 1 << 0 ) );}while(0)
#define GPIOB_REG_RESET()	do{( RCC->AHB2RSTR |= ( 1 << 1 ) ); ( RCC->AHB2RSTR &= ~( 1 << 1 ) );}while(0)
#define GPIOC_REG_RESET()	do{( RCC->AHB2RSTR |= ( 1 << 2 ) ); ( RCC->AHB2RSTR &= ~( 1 << 2 ) );}while(0)
#define GPIOD_REG_RESET()	do{( RCC->AHB2RSTR |= ( 1 << 3 ) ); ( RCC->AHB2RSTR &= ~( 1 << 3 ) );}while(0)
#define GPIOE_REG_RESET()	do{( RCC->AHB2RSTR |= ( 1 << 4 ) ); ( RCC->AHB2RSTR &= ~( 1 << 4 ) );}while(0)
#define GPIOF_REG_RESET()	do{( RCC->AHB2RSTR |= ( 1 << 5 ) ); ( RCC->AHB2RSTR &= ~( 1 << 5 ) );}while(0)
#define GPIOG_REG_RESET()	do{( RCC->AHB2RSTR |= ( 1 << 6 ) ); ( RCC->AHB2RSTR &= ~( 1 << 6 ) );}while(0)
#define GPIOH_REG_RESET()	do{( RCC->AHB2RSTR |= ( 1 << 7 ) ); ( RCC->AHB2RSTR &= ~( 1 << 7 ) );}while(0)
#define GPIOI_REG_RESET()	do{( RCC->AHB2RSTR |= ( 1 << 8 ) ); ( RCC->AHB2RSTR &= ~( 1 << 8 ) );}while(0)

//-----------------------------------------------------------------------------------

#define GPIO_BASEADDR_TO_CODE( gpioX )	( 	(gpioX == GPIOA)?0:\
											(gpioX == GPIOB)?1:\
											(gpioX == GPIOC)?2:\
											(gpioX == GPIOD)?3:\
											(gpioX == GPIOE)?4:\
											(gpioX == GPIOF)?5:\
											(gpioX == GPIOG)?6:\
											(gpioX == GPIOH)?7:0 )





/*
 * Generic MACROS
 */

#define ENABLE 	1
#define DISABLE 0

#define SET		ENABLE
#define RESET	DISABLE

#define GPIO_PIN_SET	SET
#define GPIO_PIN_RESET	RESET



#endif /* __STM32L4R5xxEXTD_H */
