/*
 * stm32l4r5xx_gpio.h
 *
 *  Created on: Apr 22, 2021
 *      Author: pratham_ovalekar
 */

#ifndef INC_STM32L4R5XX_GPIO_H_
#define INC_STM32L4R5XX_GPIO_H_

#include "stm32l4r5xxExtd.h"

/*
 * @GPIO_PIN_MODES
 * GPIO pin mode
 */
#define GPIO_MODE_IN			0			// Input mode
#define GPIO_MODE_OUT			1			// General purpose output mode
#define GPIO_MODE_ALTFN			2			// Alternate function mode
#define GPIO_MODE_ANALOG		3			// Analog mode (reset state)
#define GPIO_MODE_IT_FT			4			// Interrupt Falling edge
#define GPIO_MODE_IT_RT			5			// Interrupt Rising edge
#define GPIO_MODE_IT_RFT		6


/*
 * GPIO pin output types
 */
#define GPIO_OP_TYPE_PP			0			// Output push-pull (reset state)
#define GPIO_OP_TYPE_OD			1			// Output open-drain


/*
 * @GPIO_speed
 * GPIO pin output speed
 */
#define GPIO_SPEED_LOW			0			// Low speed
#define GPIO_SPEED_MEDIUM		1			// Medium speed
#define GPIO_SPEED_HIGH			2			// High speed
#define GPIO_SPEED_VERY_HIGH	3			// Very high speed


/*
 * @GPIO_pull
 * GPIO  pull-up/pull-down configuration options
 */
#define GPIO_NO_PUPD			0			// No pull-up, pull-down
#define GPIO_PIN_PU				1			// Pull-up
#define GPIO_PIN_PD				2			// Pull-down

/*
 * GPIO alternate function
 *
 */
#define AF0						0
#define AF1						1
#define AF2						2
#define AF3						3
#define AF4						4
#define AF5						5
#define AF6						6
#define AF7						7
#define AF8						8
#define AF9						9
#define AF10					10
#define AF11					11
#define AF12					12
#define AF13					13
#define AF14					14
#define AF15					15

/*
 * @GPIO_pins GPIO pins
 */
#define GPIO_PIN_0                 0   		// Pin 0
#define GPIO_PIN_1                 1   		// Pin 1
#define GPIO_PIN_2                 2   		// Pin 2
#define GPIO_PIN_3                 3   		// Pin 3
#define GPIO_PIN_4                 4   		// Pin 4
#define GPIO_PIN_5                 5   		// Pin 5
#define GPIO_PIN_6                 6   		// Pin 6
#define GPIO_PIN_7                 7   		// Pin 7
#define GPIO_PIN_8                 8   		// Pin 8
#define GPIO_PIN_9                 9   		// Pin 9
#define GPIO_PIN_10                10  		// Pin 10
#define GPIO_PIN_11                11  		// Pin 11
#define GPIO_PIN_12                12  		// Pin 12
#define GPIO_PIN_13                13  		// Pin 13
#define GPIO_PIN_14                14  		// Pin 14
#define GPIO_PIN_15                15  		// Pin 15



/*
 *  GPIO Init structure definition
 */
typedef struct
{
  uint8_t GPIO_PinNumber;        		/*!< Specifies the GPIO pins to be configured.
                           	   	   	   This parameter can be any value of @ref GPIO_pins */

  uint8_t GPIO_PinMode;       			/*!< Specifies the operating mode for the selected pins.
                           	   	   	    This parameter can be a value of @ref GPIO_PIN_MODES */

  uint8_t GPIO_OPType;

  uint8_t GPIO_PinSpeed;      			/*!< Specifies the speed for the selected pins.
                           	   	   	   	This parameter can be a value of @ref GPIO_speed */

  uint8_t GPIO_PinPuPdControl;			/*!< Specifies the Pull-up or Pull-Down activation for the selected pins.
										This parameter can be a value of @ref GPIO_pull */

  uint8_t GPIO_PinAlternate;  			/*!< Peripheral to be connected to the selected pins
                            			This parameter can be a value of @ref GPIOEx_Alternate_function_selection */
}GPIO_InitTypeDef;



typedef struct
{
	GPIO_TypeDef *pGPIOx;				// This holds the base address of the GPIO Port to which the pin belong
	GPIO_InitTypeDef GPIO_PinfConf;		// This structure hold GPIO Pin Config Settings

}GPIO_Handle_t;




/* GPIO Peripheral Clock Setup function	***********************************/
void GPIO_PeriClockControl( GPIO_TypeDef *pGPIOx, uint8_t EnorDi );

/* Initialization and de-initialization functions *****************************/
void GPIO_Init( GPIO_Handle_t *pGPIOHandle );	//Init A particular PIN
void GPIO_DeInit( GPIO_TypeDef *pGPIOx ) ; 		// Deinitializes the whole Port

/* IO operation functions *****************************************************/
uint8_t GPIO_ReadPin( GPIO_TypeDef *pGPIOx, uint16_t GPIO_Pin );
void GPIO_WritePin( GPIO_TypeDef *pGPIOx, uint16_t GPIO_Pin, uint8_t Value );
void GPIO_TogglePin( GPIO_TypeDef* GPIOx, uint16_t GPIO_Pin );

/* IRQ APIs	*******************************************************************/
void GPIO_IRQConfig( IRQn_Type IRQNumber, uint8_t IRQPriority, uint8_t EnOrDi );
void testInitGPIOInterrupt( void );

#endif /* INC_STM32L4R5XX_GPIO_H_ */
