/*
 * stm32l4r5xx_uart.h
 *
 *  Created on: Apr 23, 2021
 *      Author: pratham_ovalekar
 */

#ifndef INC_STM32L4R5XX_UART_H_
#define INC_STM32L4R5XX_UART_H_

#include "stm32l4r5xxExtd.h"
/* Standard includes. */
#include <stdlib.h>
#include <stdbool.h>


/*
 *@USART_Mode
 *Possible options for USART_Mode
 */
#define USART_MODE_ONLY_TX 		0
#define USART_MODE_ONLY_RX 		1
#define USART_MODE_TXRX  		2

/*
 *@USART_Baud
 *Possible options for USART_Baud
 */
#define USART_STD_BAUD_1200					1200
#define USART_STD_BAUD_2400					2400
#define USART_STD_BAUD_9600					9600
#define USART_STD_BAUD_19200 				19200
#define USART_STD_BAUD_38400 				38400
#define USART_STD_BAUD_57600 				57600
#define USART_STD_BAUD_115200 				115200
#define USART_STD_BAUD_230400 				230400
#define USART_STD_BAUD_460800 				460800
#define USART_STD_BAUD_921600 				921600
#define USART_STD_BAUD_2M 					2000000
#define SUART_STD_BAUD_3M 					3000000


/*
 *@USART_ParityControl
 *Possible options for USART_ParityControl
 */
#define USART_PARITY_DISABLE   	0
#define USART_PARITY_EN_EVEN  	1
#define USART_PARITY_EN_ODD   	2


/*
 *@USART_WordLength
 *Possible options for USART_WordLength
 *	The word length can be set to 7, 8 or 9 bits, by programming the M bits
 * (M0: bit 12 and M1:  bit 28) in the USART_CR1 register
 */
#define USART_WORDLEN_7BITS  0		//• 7-bit character length: M[1:0] = ‘10’
#define USART_WORDLEN_8BITS  1		//• 8-bit character length: M[1:0] = ‘00’
#define USART_WORDLEN_9BITS  2		//• 9-bit character length: M[1:0] = ‘01’

/*
 *@USART_NoOfStopBits
 *number of stop bits can be configured to 0.5, 1, 1.5 or 2.( USART_CR2 )
 */
#define USART_STOPBITS_1     0
#define USART_STOPBITS_0_5   1
#define USART_STOPBITS_2     2
#define USART_STOPBITS_1_5   3

/*
 *@USART_HWFlowControl
 *Possible options for USART_HWFlowControl
 */
#define USART_HW_FLOW_CTRL_NONE    	0
#define USART_HW_FLOW_CTRL_CTS    	1
#define USART_HW_FLOW_CTRL_RTS    	2
#define USART_HW_FLOW_CTRL_CTS_RTS	3


/*
 * USARTx Init/Config structure defination
 */
typedef struct
{
	uint8_t USART_Mode;
	uint32_t USART_Baud;
	uint8_t USART_NumOfStopBits;
	uint8_t USART_WordLength;
	uint8_t USART_ParityControl;
	uint8_t USART_HWFlowControl;

}USART_InitTypeDef;


/*
 * USARTx peripheral Handle structure
 */
typedef struct
{
	USART_TypeDef *pUSARTx;
	USART_InitTypeDef USART_Config;

}USART_Handle_t;



/*
 * USART flags ( ISR stands for the USART Interrupt and Status register )
 */

#define USART_FLAG_TXE 			USART_ISR_TXE_TXFNF
#define USART_FLAG_RXNE 		USART_ISR_RXNE_RXFNE
#define USART_FLAG_TC 			USART_ISR_TC


/* USART Peripheral Clock Setup function	***********************************/
void USART_PeriClockControl( USART_TypeDef *pUSARTx, uint8_t EnorDi );

/* Initialization and de-initialization functions *****************************/
void USART_Init( USART_Handle_t *pUSARTHandle );
//void USART_DeInit( USART_TypeDef *pGPIOx ) ;

/* IO operation functions *****************************************************/
void USART_SendData(USART_Handle_t *pUSARTHandle, uint8_t *pTxBuffer, uint32_t Len);
void USART_ReceiveData(USART_Handle_t *pUSARTHandle, uint8_t *pRxBuffer, uint32_t Len);


/* IRQ APIs	*******************************************************************/
void USART2_IRQConfigEnable( void );
void USART3_IRQConfigEnable( void );


void USART_EXTI_IRQHandler(uint16_t GPIO_Pin);


/*
 * Other Peripheral Control APIs
 */
void USART_PeripheralControl(USART_TypeDef *pUSARTx, uint8_t EnOrDi);

uint8_t USART_GetFlagStatus(USART_TypeDef *pUSARTx , uint32_t FlagName);
void USART_ClearFlag(USART_TypeDef *pUSARTx, uint16_t StatusFlagName);


bool USART3_BluetoothRx( uint8_t *pcRxedChar );


void  USART3_queueInit( void );
void USART3_queueReset( void );

void sendDataOnQueue( void );



#endif /* INC_STM32L4R5XX_UART_H_ */
