/*
 * at_commands.c
 *
 *  Created on: May 18, 2021
 *      Author: pratham_ovalekar
 */

#include "at_commands.h"
#include<string.h>

extern USART_Handle_t USART2Handle;
extern USART_Handle_t USART3Handle;


static AT_mode_t atCurrentMode = AT_MODE_UNKNOWN;

void parseATCommandResponse( void )
{
	size_t arrSize;
	uint8_t *array = NULL;
	uint8_t response = 0 ;

	while( array == NULL )
	{
		array = atReadLine( &arrSize );
	}

	if( arrSize > 1 )
	{
		if( strcmp((char*)array, "OK") == 0)
		{
			response = 1;
		}
		else if(strcmp((char*)array, "ERROR") == 0 )
		{
			response = 2;
		}
	}

}







uint8_t* atReadLine(size_t *atMaxBufferSize)
{
	//uint8_t inputLineBuffer[ucxhAT_INBUF_SIZE+1] = {'\0'};
	uint8_t inputLineBuffer[10+1] = {'\0'};
	uint8_t endOfLineReached;
	int bytesRead ;
	static uint8_t savedLineBuffer[ucxhAT_INBUF_SIZE+1] = {'\0'};;
	static uint8_t savedLineBufferSize = 0;
	uint16_t testVal;


	bytesRead = readBluetoothQueue( inputLineBuffer, &endOfLineReached );


	if ( (bytesRead > 0) || ( endOfLineReached == EOL_REACHED) )
	{
		// save the received message into a buffer.
		memcpy ( savedLineBuffer + savedLineBufferSize  , inputLineBuffer, (bytesRead ) );

		savedLineBufferSize = savedLineBufferSize + bytesRead;

		// If   endOfLineReached reached
		if( endOfLineReached == EOL_NOT_REACHED )
		{
			//savedLineBufferSize = 0;
			return NULL;
		}
		else
		{
			*atMaxBufferSize = savedLineBufferSize - 1;
			savedLineBuffer[*atMaxBufferSize - 1] = '\0';
			savedLineBufferSize = 0;
			//send the response command;
			return savedLineBuffer;
		}
	}
	else if( bytesRead == 0 )
	{
	  // Nothing in incoming buffer
	  return NULL;
	}
	return savedLineBuffer;
}


int readBluetoothQueue( uint8_t *outbuffer, uint8_t *eol )
{

	uint8_t *dest = outbuffer;  // Current position in output pointer
	uint8_t testVal;
	int totalBytesRead = 0;     // Total number of bytes read into the outbuffer

	*eol = EOL_NOT_REACHED;

	if( USART3_BluetoothRx( &testVal ) == true )
	{
		if( testVal == NL )
		{
			totalBytesRead = totalBytesRead + 1;
			*eol = EOL_REACHED;

		}
		else
		{
			*eol = EOL_NOT_REACHED;
			*dest = testVal;
			totalBytesRead = totalBytesRead + 1;
			dest = dest + 1;
		}

	}

	return totalBytesRead;
}




/*!
 *  \brief Send AT command to the ANNA-B12 module over the USART 3
 *
 *  \param pTxBuffer
 *  String of Data/ AT command to be sent to the ANNA-B12 module
 */
void sendATCommand(  char *pTxBuffer )
{
	uint32_t length;
	length = strlen(pTxBuffer);
	USART_SendData( &USART3Handle, (uint8_t*)pTxBuffer, length );
}





bool enterAtMode(void)
{

  sendATCommand( AT_CMD_LEAVE_DATA_MODE );

  //sendATCommand( AT_CMD_LEAVE_DATA_MODE );
  //sendATCommand( AT_CMD_LEAVE_DATA_MODE );

  //for( uint32_t i =0; i < 500000; i++ );

 // sendATCommand( AT_COMMAND );
  /*
   *  parse the response
  if (ucxhAT_waitForOk(-1) != ucxhAT_RESPONSE_OK) {  // If failed, module is probably in EDM mode.
    atCurrentMode = ucxhAT_MODE_UNKNOWN;
    return false;
  }
  */
  atCurrentMode = AT_MODE_AT;
  return true;
}



bool enterDataMode(void)
{

	sendATCommand( AT_CMD_ENTER_DATA_MODE );
	// usleep(1100000);  // 1.1s

	/*
	 * parse the response
	if (ucxhAT_waitForOk(-1) != ucxhAT_RESPONSE_OK) {  // If failed, module is probably in data or EDM mode.
	atCurrentMode = ucxhAT_MODE_UNKNOWN;
	return false;
	}
	*/

	//usleep(55000); // 50 ms minimum before transmitting data

	atCurrentMode = AT_MODE_DATA;
	return true;
}


AT_mode_t atGetCurrentMode(void)
{
  return atCurrentMode;
}
