/*
 * bluetooth.c
 *
 *  Created on: Jun 9, 2021
 *      Author: pratham_ovalekar
 */

#include 	"stm32l4r5xx_gpio.h"
#include 	"bluetooth.h"
#include	<string.h>
#include 	"common.h"

extern USART_Handle_t USART2Handle;
extern USART_Handle_t USART3Handle;


static uint8_t DSR_interruptOccured = 0;


void delayBLE ( void )
{
	for( uint32_t i =0; i < 20000; i++ );
}


/*!
 *  \brief Configure the DSR GPIO
 *  Interrupt : Low to high (Rising edge)
 *
 */
void configureDSR_GPIO_pin( void )
{
	// IRQ configuration


	GPIO_Handle_t gpio = {0};
	GPIO_PeriClockControl( BLUETOOTH_DRS_PORT , ENABLE );

	gpio.pGPIOx = BLUETOOTH_DRS_PORT;
	gpio.GPIO_PinfConf.GPIO_PinNumber = BLUETOOTH_DRS_PIN;
	gpio.GPIO_PinfConf.GPIO_PinMode = GPIO_MODE_IT_RT;
	gpio.GPIO_PinfConf.GPIO_PinSpeed = GPIO_SPEED_HIGH;
	gpio.GPIO_PinfConf.GPIO_PinPuPdControl = GPIO_NO_PUPD;
	GPIO_Init( &gpio );

	GPIO_IRQConfig( BLUETOOTH_DRS_IRQ_NUM, 6, ENABLE );

}


/*!
 *  \brief Initialize the Bluetooth Module
 *
 */
void initBluetooth( void )
{


	//configureDSR_GPIO_pin();

	enterAtMode();
	delayBLE();


	sendATCommand( AT_COMMAND );
	delayBLE();

	sendATCommand( AT_ECHO_OFF );
	//parseATCommandResponse();
	//parseATCommandResponse();
	//parseATCommandResponse();



	delayBLE();
	sendATCommand( AT_MANU_IDENTI );
	//parseATCommandResponse();
	//parseATCommandResponse();
	//parseATCommandResponse();


	delayBLE();
	sendATCommand( AT_DTS_MODE_2 );

	delayBLE();
	sendATCommand( AT_MODEL_IDENTI );
	//parseATCommandResponse();
	//parseATCommandResponse();
	//parseATCommandResponse();

	delayBLE();
	sendATCommand( AT_SOFT_VER_ID );
	//parseATCommandResponse();
	//parseATCommandResponse();
	//parseATCommandResponse();

	delayBLE();
	sendATCommand( AT_SERIAL_NUM );
	//parseATCommandResponse();
	//parseATCommandResponse();
	//parseATCommandResponse();

	delayBLE();
	sendATCommand( AT_DISCOV );
	//parseATCommandResponse();
	//parseATCommandResponse();

	delayBLE();
	sendATCommand( AT_DISCOV_Q );
	//parseATCommandResponse();
	//parseATCommandResponse();
	//parseATCommandResponse();

	delayBLE();
	sendATCommand( AT_CONECT );
	//parseATCommandResponse();
	//parseATCommandResponse();

	delayBLE();
	sendATCommand( AT_CONECT_Q );
	//parseATCommandResponse();
	//parseATCommandResponse();
	//parseATCommandResponse();

	// Bluetooth low energy role:: Setting the device into BLE Peripheral.
	delayBLE();
	sendATCommand( AT_BLE_ROLE_1 );
	delayBLE();
	sendATCommand( AT_BLE_ROLE_Q );



	// First set the security and then set the pairing mode.
	// Because setting the Security will disable the pairing
	delayBLE();
	sendATCommand( AT_SECURITY_MODE_1 );
	delayBLE();
	sendATCommand( AT_SECURITY_MODE_Q );


	delayBLE();
	sendATCommand( AT_PAIRING_MODE_YES );
	delayBLE();
	sendATCommand( AT_PAIRING_MODE_Q );
	delayBLE();





}



void manageBLEComm( void )
{
	uint8_t *array = NULL;
	size_t arrSize;
	static uint8_t dataMode = 0;
	uint8_t rxData;

	if( dataMode == 0 )
	{

		array = atReadLine( &arrSize );

		if( array != NULL)
		{
			USART_SendData( &USART2Handle, array, strlen((char*)array) );
			rxData = NL;
			USART_SendData( &USART2Handle, &rxData, 1 );
			rxData = CR;
			USART_SendData( &USART2Handle, &rxData, 1 );

			if( strncmp((char*)array, "+UUDPC:1,1,4",12) == 0 )
			{
				enterDataMode();
				dataMode = 1;
			}
			else if( strcmp( (char*)array, "+UUDPD:1" ) == 0 )
			{
				enterAtMode();
				dataMode = 0;
			}
		}
	}
	else if( dataMode == 1 )
	{
		if( USART3_BluetoothRx( &rxData ) == true )
		{
			//USART_SendData( &USART2Handle, &rxData, 1 );
			USART_SendData( &USART3Handle, &rxData, 1 );
			//rxData = NL;
			//USART_SendData( &USART2Handle, &rxData, 1 );
			//rxData = CR;
			//USART_SendData( &USART2Handle, &rxData, 1 );
			// Parse the data received
		}
	}

	// Check if there was on interrupt generated on the DSR line:
	//	This
	//ASSERT : DSR line when at least one remote peer is connected.(logical 0)
	//DEASSERT:  DSR line when no remote peers are connected.(logical 1)
	// If no remote peers connected then change the module mode to command mode.
	if( DSR_interruptOccured == 1 )
	{
		DSR_interruptOccured = 0;
		enterAtMode();
		dataMode = 0;

	}


}



void dsrInterruptOccured( void )
{
	DSR_interruptOccured = 1;
}

