/*
 * stm32l4r5xx_decoder.c
 *
 *  Created on: May 5, 2021
 *      Author: pratham_ovalekar
 */


#include "stm32l4r5xx_decoder.h"


void DecoderInit( Decoder_Handle_t *decHandle )
{
	uint32_t testReg = 0;
	/* Set the encoder/decoder Mode */
	testReg |= TIM_SMCR_SMS_ENCODER_3;
	decHandle->DecoderBase->SMCR |= testReg;

	/* Set the TI1 and the TI2 Polarities */
	// CC1NP=0, CC1P=0: non-inverted/rising edge. The circuit is sensitive to
	// TIxFP1 rising edge 	(capture or trigger operations in reset, external
	// clock or trigger mode), 	TIxFP1 is not inverted (trigger operation in
	// gated mode or encoder mode).
	testReg = 0;
	testReg &= ~(1 << TIM_CCER_CC1P_Pos);
	testReg &= ~(1 << TIM_CCER_CC1NP_Pos);
	testReg &= ~(1 << TIM_CCER_CC2P_Pos);
	testReg &= ~(1 << TIM_CCER_CC2NP_Pos);
	decHandle->DecoderBase->CCER |= testReg;

	// input filter


	//
	testReg = 0;
	decHandle->DecoderBase->CCMR1 |= TIM_CCMR1_CC1S_Pos;
	decHandle->DecoderBase->CCMR1 &= ~(TIM_CCMR1_CC1S_Pos + 1);

	decHandle->DecoderBase->CCMR1 |= TIM_CCMR1_CC2S_Pos;
	decHandle->DecoderBase->CCMR1 &= ~(TIM_CCMR1_CC2S_Pos + 1);


	decHandle->DecoderBase->CR1 &= ~ ( 0x01UL << TIM_CR1_CKD_Pos );
	decHandle->DecoderBase->CR1 &= ~ ( 0x01UL << (TIM_CR1_CKD_Pos + 1) );

	decHandle->DecoderBase->CR1 &= ~( TIM_CR1_ARPE );



	// TIMx auto-reload register


}

void DecoderGetCount( Decoder_Handle_t *decHandle )
{

}

uint32_t DecoderGetDirection( Decoder_Handle_t *decHandle )
{
	/*
	 * . Depending on the sequence the counter counts up or down, the DIR bit
	 * in the TIMx_CR1 register is modified by hardware accordingly. The DIR bit
	 * is calculated at each transition on any input (TI1 or TI2), whatever the
	 * counter is counting on TI1 only, TI2 only or both TI1 and TI2.
	 */
	uint32_t value;

	//

	value = decHandle->DecoderBase->CR1;

	value = ( value >> TIM_CR1_DIR_Pos ) & 1;
	return value;

}


void DecoderEnable( Decoder_Handle_t *decHandle )
{
	decHandle->DecoderBase->CR1 |= TIM_CR1_CEN;
}
