/*
 * stm32l4r5xx_gpio.c
 *
 *  Created on: Apr 22, 2021
 *      Author: pratham_ovalekar
 */

#include "stm32l4r5xx_gpio.h"




/*!
 *  \brief GPIO Peripheral Clock Setup function
 *
 *  \param pGPIOx
 *  Base Address of the GPIO
 *
 *  \param EnorDi
 *  ENABLE or DISABLE macros
 *
 */
void GPIO_PeriClockControl( GPIO_TypeDef *pGPIOx, uint8_t EnorDi )
{
	if ( EnorDi  == ENABLE )
	{
		if( pGPIOx == GPIOA )
			GPIOA_PCLK_EN();
		else if( pGPIOx == GPIOB )
			GPIOB_PCLK_EN();
		else if( pGPIOx == GPIOC )
			GPIOC_PCLK_EN();
		else if( pGPIOx == GPIOD )
			GPIOD_PCLK_EN();
		else if( pGPIOx == GPIOE )
			GPIOE_PCLK_EN();
		else if( pGPIOx == GPIOF )
			GPIOF_PCLK_EN();
		else if( pGPIOx == GPIOG )
			GPIOG_PCLK_EN();
		else if( pGPIOx == GPIOH )
			GPIOH_PCLK_EN();
		else if( pGPIOx == GPIOI )
			GPIOI_PCLK_EN();
	}
	else if( EnorDi == DISABLE )
	{
		if( pGPIOx == GPIOA )
			GPIOA_PCLK_DI();
		else if( pGPIOx == GPIOB )
			GPIOB_PCLK_DI();
		else if( pGPIOx == GPIOC )
			GPIOC_PCLK_DI();
		else if( pGPIOx == GPIOD )
			GPIOD_PCLK_DI();
		else if( pGPIOx == GPIOE )
			GPIOE_PCLK_DI();
		else if( pGPIOx == GPIOF )
			GPIOF_PCLK_DI();
		else if( pGPIOx == GPIOG )
			GPIOG_PCLK_DI();
		else if( pGPIOx == GPIOH )
			GPIOH_PCLK_DI();
		else if( pGPIOx == GPIOI )
			GPIOI_PCLK_DI();

	}
}




/*!
 *  \brief Initialization GPIO Pin
 *
 *  \param pGPIOHandle
 *	Handle of the GPOI struct that has the base address to the PORT and the initializing structure
 *
 */
void GPIO_Init( GPIO_Handle_t *pGPIOHandle )
{
	uint32_t temp, temp1, temp2, portcode;

	temp = 0;
	// Configure Mode
	if( pGPIOHandle->GPIO_PinfConf.GPIO_PinMode <= GPIO_MODE_ANALOG )
	{
		//Non Interrupt based
		temp = pGPIOHandle->GPIO_PinfConf.GPIO_PinMode  << ( 2 * pGPIOHandle->GPIO_PinfConf.GPIO_PinNumber );
		pGPIOHandle->pGPIOx->MODER &= ~( 0x3 << ( 2 * pGPIOHandle->GPIO_PinfConf.GPIO_PinNumber ) ); //Clearing
 		pGPIOHandle->pGPIOx->MODER |= temp; //Setting
	}
	else
	{
		//Interrupt Based
		switch(pGPIOHandle->GPIO_PinfConf.GPIO_PinMode)
		{
			case GPIO_MODE_IT_FT:
				// Config falling trigger register FTSR
				EXTI->FTSR1 |= ( 1 << pGPIOHandle->GPIO_PinfConf.GPIO_PinNumber);
				// Clear the RTSR bit
				EXTI->RTSR1 &= ~( 1 << pGPIOHandle->GPIO_PinfConf.GPIO_PinNumber);
				break;

			case GPIO_MODE_IT_RT:
				//config RTSR
				EXTI->RTSR1 |= ( 1 << pGPIOHandle->GPIO_PinfConf.GPIO_PinNumber);
				// Clear the FTSR bit
				EXTI->FTSR1 &= ~( 1 << pGPIOHandle->GPIO_PinfConf.GPIO_PinNumber);
				break;

			case GPIO_MODE_IT_RFT:
				//config RTSR and FTSR
				EXTI->FTSR1 |= ( 1 << pGPIOHandle->GPIO_PinfConf.GPIO_PinNumber);
				EXTI->RTSR1 |= ( 1 << pGPIOHandle->GPIO_PinfConf.GPIO_PinNumber);

				break;
		}

		// configure the gpio port selection in stscfg_exticr
		// Enable the syscfg clock
		SYSCONFG_PCLK_EN();

		temp1 = pGPIOHandle->GPIO_PinfConf.GPIO_PinNumber / 4;
		temp2 = pGPIOHandle->GPIO_PinfConf.GPIO_PinNumber % 4;

		portcode = GPIO_BASEADDR_TO_CODE(pGPIOHandle->pGPIOx);

		//SYSCFG->EXTICR[temp1] &= ~(0xF << (4 * temp2));
		SYSCFG->EXTICR[temp1] |= portcode << ( temp2 * 4);


		//enable the exti interrupt delivery using IMR Interrupt mask register 1
		EXTI->IMR1 |= ( 1 << pGPIOHandle->GPIO_PinfConf.GPIO_PinNumber);

	}
	temp = 0;



	//Configure the OP type
	temp =  ( pGPIOHandle->GPIO_PinfConf.GPIO_OPType  <<  pGPIOHandle->GPIO_PinfConf.GPIO_PinNumber );
	pGPIOHandle->pGPIOx->OTYPER &= ~( 0x1 << pGPIOHandle->GPIO_PinfConf.GPIO_PinNumber ); //Clearing
	pGPIOHandle->pGPIOx->OTYPER |= temp;
	temp = 0;


	// Configure Speed
	temp = ( pGPIOHandle->GPIO_PinfConf.GPIO_PinSpeed  << ( 2 * pGPIOHandle->GPIO_PinfConf.GPIO_PinNumber ) );
	pGPIOHandle->pGPIOx->OSPEEDR &= ~( 0x3 << ( 2 * pGPIOHandle->GPIO_PinfConf.GPIO_PinNumber) ); //Clearing
	pGPIOHandle->pGPIOx->OSPEEDR |= temp;
	temp = 0;


	//Configure Pull-up or Pull-Down
	temp = ( pGPIOHandle->GPIO_PinfConf.GPIO_PinPuPdControl  << ( 2 * pGPIOHandle->GPIO_PinfConf.GPIO_PinNumber ) );
	pGPIOHandle->pGPIOx->PUPDR &= ~( 0x3 << ( 2 * pGPIOHandle->GPIO_PinfConf.GPIO_PinNumber) ); //Clearing
	pGPIOHandle->pGPIOx->PUPDR |= temp;



	//Configure Alternate Functionality
	if( pGPIOHandle->GPIO_PinfConf.GPIO_PinMode == GPIO_MODE_ALTFN )
	{
		uint8_t temp1,temp2;

		temp1 = pGPIOHandle->GPIO_PinfConf.GPIO_PinNumber / 8;
		temp2 = pGPIOHandle->GPIO_PinfConf.GPIO_PinNumber % 8;

		//pGPIOHandle->pGPIOx->AFR[temp1] &= ~(0xF << ( 4 * temp2 ) ); //clearing
		pGPIOHandle->pGPIOx->AFR[temp1] |= (  pGPIOHandle->GPIO_PinfConf.GPIO_PinAlternate << ( 4 * temp2) );


	}

}




/*!
 *  \brief De-Initialization the whole GPIO Port
 *
 *  \param pGPIOx
 *  Base Address of the GPIO
 *
 */
void GPIO_DeInit( GPIO_TypeDef *pGPIOx )
{
	if( pGPIOx == GPIOA )
		GPIOA_REG_RESET();
	else if( pGPIOx == GPIOB )
		GPIOB_REG_RESET();
	else if( pGPIOx == GPIOC )
		GPIOC_REG_RESET();
	else if( pGPIOx == GPIOD )
		GPIOD_REG_RESET();
	else if( pGPIOx == GPIOE )
		GPIOE_REG_RESET();
	else if( pGPIOx == GPIOF )
		GPIOF_REG_RESET();
	else if( pGPIOx == GPIOG )
		GPIOG_REG_RESET();
	else if( pGPIOx == GPIOH )
		GPIOH_REG_RESET();
	else if( pGPIOx == GPIOI )
		GPIOI_REG_RESET();
}



/* IO operation functions *****************************************************/
/*!
 *  \brief Read the GPIO Pin
 *
 *  \param pGPIOx
 *  Base Address of the GPIO
 *
 *  \param GPIO_Pin
 *  Pin Number of the GPIO
 *
 */
uint8_t GPIO_ReadPin( GPIO_TypeDef *pGPIOx, uint16_t GPIO_Pin )
{
	uint8_t pinVal;

	pinVal = (uint8_t) ( ( pGPIOx->IDR >> GPIO_Pin ) & 0x00000001 );

	return pinVal;
}

/*!
 *  \brief Write to the GPIO Pin
 *
 *  \param pGPIOx
 *  Base Address of the GPIO
 *
 *  \param GPIO_Pin
 *  Pin Number of the GPIO
 *
 *  \param Value
 *  GPIO_PIN_SET or GPIO_PIN_RESET Macro
 *
 */
void GPIO_WritePin( GPIO_TypeDef *pGPIOx, uint16_t GPIO_Pin, uint8_t Value )
{
	if( Value == GPIO_PIN_SET )
	{
		pGPIOx->ODR |= (1 << GPIO_Pin);
	}
	else
	{
		pGPIOx->ODR &= ~(1 << GPIO_Pin);
	}

}

/*!
 *  \brief Toggle the GPIO Pin
 *
 *  \param pGPIOx
 *  Base Address of the GPIO
 *
 *  \param GPIO_Pin
 *  Pin Number of the GPIO
 *
 */
void GPIO_TogglePin( GPIO_TypeDef* GPIOx, uint16_t GPIO_Pin )
{
	GPIOx->ODR ^= ( 1 << GPIO_Pin );
}



/* IRQ APIs	*******************************************************************/
void GPIO_IRQConfig( IRQn_Type IRQNumber, uint8_t IRQPriority, uint8_t EnOrDi )
{

	// Set Interrupt Priority
	NVIC_SetPriority(IRQNumber, IRQPriority );

	// Enable or Disable

	if( EnOrDi == ENABLE )
		NVIC_EnableIRQ( IRQNumber );
	else
		NVIC_DisableIRQ( IRQNumber );

}



void testInitGPIOInterrupt( void )
{

	// PC 13
	uint8_t pinNumber;
	uint32_t temp;

	pinNumber = 13;

	RCC->AHB2ENR |= ( 1 << RCC_AHB2ENR_GPIOCEN_Pos );

	RCC->APB2ENR  |= ( 1 << RCC_APB2ENR_SYSCFGEN_Pos );
	RCC->APB2ENR;  // Read back the (volatile) RCC register as a short delay.
	RCC->APB2ENR;  // Read back the (volatile) RCC register as a short delay.
	SYSCFG->EXTICR[(pinNumber/4)] &= ~(0xF << ((pinNumber % 4) * 4));
	SYSCFG->EXTICR[(pinNumber/4)] |=  (0x2 << ((pinNumber % 4) * 4));


	//enable the exti interrupt delivery using IMR Interrupt mask register 1
	EXTI->IMR1 |= ( 1 << pinNumber );
	//  Enable the 'rising edge' trigger
	EXTI->RTSR1 |= ( 1 << pinNumber );
	// Disable the 'falling edge' trigger
	EXTI->FTSR1 &= ~( 1 << pinNumber );


	GPIOC->MODER &= ~( 0x3 << ( pinNumber*2 ) );
	GPIOC->PUPDR &= ~( 0x3 << ( pinNumber*2 ) );
	GPIOC->PUPDR |= ( 0x0 << ( pinNumber*2 ) );

	// Enable IRQ
	NVIC_SetPriority( EXTI15_10_IRQn, 6 );
	NVIC_EnableIRQ ( EXTI15_10_IRQn );

}
