/*
 * stm32l4r5xx_uartISR.c
 *
 *  Created on: May 26, 2021
 *      Author: pratham_ovalekar
 */

#include "stm32l4r5xx_uart.h"

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "queue.h"
#include "projdefs.h"



extern USART_Handle_t USART2Handle;


/* Queues used to hold received characters */
static QueueHandle_t USART3_RxQueue;



/*!
 *  \brief Interrupt handler for the UART 2.
 *
 */

void USART2_IRQHandler( void )
{
	uint8_t console_rx_byte;
	if ( USART2->ISR & USART_ISR_RXNE_RXFNE )
	{
		console_rx_byte = USART2->RDR;
		// Yet to be implemented as UART2 is just used for debug as of now
	}
}


/*!
 *  \brief Interrupt handler for the UART 3.
 *   Bluetooth is connected to UART 3.
 *
 */

void USART3_IRQHandler( void )
{
	uint8_t console_rx_byte;

	if ( USART3->ISR & USART_ISR_RXNE_RXFNE )
	{
		// Copy new data into the buffer.
		console_rx_byte = USART3->RDR;

		//USART_SendData( &USART2Handle, &console_rx_byte, 2 );
		xQueueSendFromISR( USART3_RxQueue, &console_rx_byte, 0 );

	}

}


/*!
 *  \brief Create a RX queue for receiving messages received from the UART3
 *  Bluetooth.
 *
 *  \param uxQueueLength
 *  Length of the queue
 *
 *  \param pxRxedChars
 *	Reference to the
 *
 */
void USART3_SerialISRCreateQueues( unsigned portBASE_TYPE uxQueueLength,
		QueueHandle_t *pxRxedChars )
{
	/* Create the queues used to hold Rx characters. */
	USART3_RxQueue = xQueueCreate( uxQueueLength,
			sizeof( uint8_t ) );

	/* Pass back a reference to the queues so the serial API file can
	receive characters. */
	*pxRxedChars = USART3_RxQueue;
}



